@@ -32,13 +32,12 @@
 #include "gnome-vfs-result.h"
 #include "gnome-vfs-private-utils.h"
 #include "gnome-vfs-utils.h"
-#include <bonobo-activation/bonobo-activation-activate.h>
+
 #include <gconf/gconf-client.h>
 #include <stdio.h>
 #include <string.h>
 #include <glib/gi18n-lib.h>
-
-#define GCONF_DEFAULT_VIEWER_EXEC_PATH   "/desktop/gnome/applications/component_viewer/exec"
+#include <gmodule.h>
 
 #define MIXED_API_WARNING "Cannot call %s with a GNOMEVFSMimeApplication structure "\
 			  "constructed by the deprecated application registry", \
@@ -51,6 +50,7 @@ struct _GnomeVFSMimeApplicationPrivate
 	char *icon;
 	char *exec;
 	char *binary_name;
+	char *path;
 	gboolean supports_uris;
 	gboolean startup_notification;
 	char *startup_wm_class;
@@ -58,15 +58,10 @@ struct _GnomeVFSMimeApplicationPrivate
 
 extern GList * _gnome_vfs_configuration_get_methods_list (void);
 
-static GnomeVFSResult expand_application_parameters              (const char               *exec,
+static GnomeVFSResult expand_application_parameters              (GnomeVFSMimeApplication *application,
 								  GList                   **uri_list,
 								  int                      *argc,
 								  char                   ***argv);
-static GnomeVFSResult expand_component_parameters                (gpointer                  action,
-								  GList                    *uris,
-								  int                      *argc,
-								  char                   ***argv);
-static GList *        Bonobo_ServerInfoList_to_ServerInfo_g_list (Bonobo_ServerInfoList    *info_list);
 static GList *        copy_str_list                              (GList                    *string_list);
 
 
@@ -116,8 +111,7 @@ gnome_vfs_mime_set_description (const char *mime_type, const char *description)
  * Query the MIME database for the type of action to be performed on @mime_type.
  * 
  * Deprecated: This function does not work with the new mime system.
- * It always returns none except for directories where component is
- * returned
+ * It always returns none
  *
  * Return value: The type of action to be performed on a file of 
  * MIME type @mime_type by default.
@@ -125,18 +119,7 @@ gnome_vfs_mime_set_description (const char *mime_type, const char *description)
 GnomeVFSMimeActionType
 gnome_vfs_mime_get_default_action_type (const char *mime_type)
 {
-	const char *action_type_string;
-
-	action_type_string = gnome_vfs_mime_get_value (mime_type, "default_action_type");
-
-	if (action_type_string != NULL && g_ascii_strcasecmp (action_type_string, "application") == 0) {
-		return GNOME_VFS_MIME_ACTION_TYPE_APPLICATION;
-	} else if (action_type_string != NULL && g_ascii_strcasecmp (action_type_string, "component") == 0) {
-		return GNOME_VFS_MIME_ACTION_TYPE_COMPONENT;
-	} else {
-		return GNOME_VFS_MIME_ACTION_TYPE_NONE;
-	}
-
+	return GNOME_VFS_MIME_ACTION_TYPE_NONE;
 }
 
 /**
@@ -157,31 +140,12 @@ gnome_vfs_mime_get_default_action (const char *mime_type)
 
 	action = g_new0 (GnomeVFSMimeAction, 1);
 
-	action->action_type = gnome_vfs_mime_get_default_action_type (mime_type);
-
-	switch (action->action_type) {
-	case GNOME_VFS_MIME_ACTION_TYPE_APPLICATION:
-		action->action.application = 
-			gnome_vfs_mime_get_default_application (mime_type);
-		if (action->action.application == NULL) {
-			g_free (action);
-			action = NULL;
-		}
-		break;
-	case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT:
-		action->action.component = 
-			gnome_vfs_mime_get_default_component (mime_type);
-		if (action->action.component == NULL) {
-			g_free (action);
-			action = NULL;
-		}
-		break;
-	case GNOME_VFS_MIME_ACTION_TYPE_NONE:
+	action->action_type = GNOME_VFS_MIME_ACTION_TYPE_APPLICATION;
+	action->action.application = 
+		gnome_vfs_mime_get_default_application (mime_type);
+	if (action->action.application == NULL) {
 		g_free (action);
 		action = NULL;
-		break;
-	default:
-		g_assert_not_reached ();
 	}
 
 	return action;
@@ -193,42 +157,34 @@ gnome_vfs_mime_get_default_action (const char *mime_type)
  * 
  * Query the MIME database for the application to be executed on files of MIME type
  * @mime_type by default.
+ *
+ * If you know the actual uri of the file you should use gnome_vfs_mime_get_default_application_for_uri
+ * instead, as it will then be able to pick a better app. For instance it won't pick
+ * an app that claims to only handle local files for a remote uri.
  * 
  * Return value: a #GnomeVFSMimeApplication representing the default handler of @mime_type.
  */
 GnomeVFSMimeApplication *
 gnome_vfs_mime_get_default_application (const char *mime_type)
 {
-	char *default_application_id;
-	GnomeVFSMimeApplication *default_application;
-	GList *list;
+	GnomeVFSMimeApplication *app;
+	GList *applications, *l;
 
-	default_application = NULL;
+	app = NULL;
+	
+	applications = gnome_vfs_mime_get_all_desktop_entries (mime_type);
+	for (l = applications; l != NULL; l = l->next) {
+		app = gnome_vfs_mime_application_new_from_id (l->data);
 
-	/* First, try the default for the mime type */
-	default_application_id = gnome_vfs_mime_get_default_desktop_entry (mime_type);
-	if (default_application_id != NULL
-	    && default_application_id[0] != '\0') {
-		default_application =
-			gnome_vfs_mime_application_new_from_id (default_application_id);
-		g_free (default_application_id);
-	}
-
-	if (default_application == NULL) {
-
-		/* Failing that, try something from the complete list */
-		list = gnome_vfs_mime_get_all_desktop_entries (mime_type);
-
-		if (list != NULL) {
-			default_application = 
-				gnome_vfs_mime_application_new_from_id ((char *) list->data);
-
-			g_list_foreach (list, (GFunc) g_free, NULL);
-			g_list_free (list);
+		if (app != NULL) {
+			break;
 		}
 	}
+	
+	g_list_foreach (applications, (GFunc) g_free, NULL);
+	g_list_free (applications);
 
-	return default_application;
+	return app;
 }
 
 /**
@@ -327,86 +283,6 @@ gnome_vfs_mime_set_can_be_executable (const char *mime_type, gboolean new_value)
 	return GNOME_VFS_ERROR_DEPRECATED_FUNCTION;
 }
 
-/**
- * gnome_vfs_mime_get_default_component:
- * @mime_type: a const char * containing a mime type, e.g. "image/png".
- * 
- * Query the MIME database for the default Bonobo component to be activated to 
- * view files of MIME type @mime_type.
- * 
- * Return value: a #Bonobo_ServerInfo * representing the OAF server to be activated
- * to get a reference to the proper component.
- */
-Bonobo_ServerInfo *
-gnome_vfs_mime_get_default_component (const char *mime_type)
-{
-	const char *default_component_iid;
-	Bonobo_ServerInfoList *info_list;
-	Bonobo_ServerInfo *default_component;
-	CORBA_Environment ev;
-	char *supertype;
-	char *query;
-	char *sort[5];
-
-	if (mime_type == NULL) {
-		return NULL;
-	}
-
-	CORBA_exception_init (&ev);
-
-	supertype = gnome_vfs_get_supertype_from_mime_type (mime_type);
-
-	/* Find a component that supports either the exact mime type,
-           the supertype, or all mime types. */
-
-	/* First try the component specified in the mime database, if available. 
-	   gnome_vfs_mime_get_value looks up the value for the mime type and the supertype.  */
-	default_component_iid = gnome_vfs_mime_get_value
-		(mime_type, "default_component_iid");
-
-	query = g_strconcat ("bonobo:supported_mime_types.has_one (['", mime_type, 
-			     "', '", supertype,
-			     "', '*'])", NULL);
-
-
-	if (default_component_iid != NULL) {
-		sort[0] = g_strconcat ("iid == '", default_component_iid, "'", NULL);
-	} else {
-		sort[0] = g_strdup ("true");
-	}
-
-	/* Prefer something that matches the exact type to something
-           that matches the supertype */
-	sort[1] = g_strconcat ("bonobo:supported_mime_types.has ('", mime_type, "')", NULL);
-
-	/* Prefer something that matches the supertype to something that matches `*' */
-	sort[2] = g_strconcat ("bonobo:supported_mime_types.has ('", supertype, "')", NULL);
-
-	sort[3] = g_strdup ("name");
-	sort[4] = NULL;
-
-	info_list = bonobo_activation_query (query, sort, &ev);
-	
-	default_component = NULL;
-	if (ev._major == CORBA_NO_EXCEPTION) {
-		if (info_list != NULL && info_list->_length > 0) {
-			default_component = Bonobo_ServerInfo_duplicate (&info_list->_buffer[0]);
-		}
-		CORBA_free (info_list);
-	}
-
-	g_free (supertype);
-	g_free (query);
-	g_free (sort[0]);
-	g_free (sort[1]);
-	g_free (sort[2]);
-	g_free (sort[3]);
-
-	CORBA_exception_free (&ev);
-
-	return default_component;
-}
-
 /**
  * gnome_vfs_mime_get_short_list_applications:
  * @mime_type: a const char * containing a mime type, e.g. "image/png".
@@ -427,79 +303,6 @@ gnome_vfs_mime_get_short_list_applications (const char *mime_type)
 	return gnome_vfs_mime_get_all_applications (mime_type);
 }
 
-
-/**
- * gnome_vfs_mime_get_short_list_components:
- * @mime_type: a const char * containing a mime type, e.g. "image/png".
- * 
- * Return an unsorted sorted list of #Bonobo_ServerInfo * data structures for the
- * @mime_type. gnome-vfs no longer supports the concept of a "short
- * list" of applications that the user might be interested in.
- * 
- * Return value: a #GList * where the elements are #Bonobo_ServerInfo *
- * representing various components to display in the short list for @mime_type.
- *
- * Deprecated: Use gnome_vfs_mime_get_all_components() instead.
- */ 
-GList *
-gnome_vfs_mime_get_short_list_components (const char *mime_type)
-{
-	Bonobo_ServerInfoList *info_list;
-	GList *components_list;
-	CORBA_Environment ev;
-	char *supertype;
-	char *query;
-	char *sort[4];
-
-	if (mime_type == NULL) {
-		return NULL;
-	}
-
-	CORBA_exception_init (&ev);
-
-	/* Find a component that supports either the exact mime type,
-           the supertype, or all mime types. */
-
-	/* FIXME bugzilla.eazel.com 1142: should probably check for
-           the right interfaces too. Also slightly semantically
-           different from nautilus in other tiny ways.
-	*/
-	supertype = gnome_vfs_get_supertype_from_mime_type (mime_type);
-	query = g_strconcat ("bonobo:supported_mime_types.has_one (['", mime_type, 
-			     "', '", supertype,
-			     "', '*'])", NULL);
-	g_free (supertype);
-	
-        /* Prefer something that matches the exact type to something
-           that matches the supertype */
-	sort[0] = g_strconcat ("bonobo:supported_mime_types.has ('", mime_type, "')", NULL);
-
-	/* Prefer something that matches the supertype to something that matches `*' */
-	sort[1] = g_strconcat ("bonobo:supported_mime_types.has ('", supertype, "')", NULL);
-
-	sort[2] = g_strdup ("name");
-	sort[3] = NULL;
-
-	info_list = bonobo_activation_query (query, sort, &ev);
-	
-	if (ev._major == CORBA_NO_EXCEPTION) {
-		components_list = Bonobo_ServerInfoList_to_ServerInfo_g_list (info_list);
-		CORBA_free (info_list);
-	} else {
-		components_list = NULL;
-	}
-
-	g_free (query);
-	g_free (sort[0]);
-	g_free (sort[1]);
-	g_free (sort[2]);
-
-	CORBA_exception_free (&ev);
-
-	return components_list;
-}
-
-
 /**
  * gnome_vfs_mime_get_all_applications:
  * @mime_type: a const char * containing a mime type, e.g. "image/png".
@@ -543,66 +346,6 @@ gnome_vfs_mime_get_all_applications (const char *mime_type)
 	return applications;
 }
 
-/**
- * gnome_vfs_mime_get_all_components:
- * @mime_type: a const char * containing a mime type, e.g. "image/png".
- * 
- * Return an alphabetically sorted list of #Bonobo_ServerInfo
- * data structures representing all Bonobo components registered
- * to handle files of @mime_type (and supertypes).
- * 
- * Return value: a #GList * where the elements are #Bonobo_ServerInfo *
- * representing components that can handle @mime_type.
- */ 
-GList *
-gnome_vfs_mime_get_all_components (const char *mime_type)
-{
-	Bonobo_ServerInfoList *info_list;
-	GList *components_list;
-	CORBA_Environment ev;
-	char *supertype;
-	char *query;
-	char *sort[2];
-
-	if (mime_type == NULL) {
-		return NULL;
-	}
-
-	CORBA_exception_init (&ev);
-
-	/* Find a component that supports either the exact mime type,
-           the supertype, or all mime types. */
-
-	/* FIXME bugzilla.eazel.com 1142: should probably check for
-           the right interfaces too. Also slightly semantically
-           different from nautilus in other tiny ways.
-	*/
-	supertype = gnome_vfs_get_supertype_from_mime_type (mime_type);
-	query = g_strconcat ("bonobo:supported_mime_types.has_one (['", mime_type, 
-			     "', '", supertype,
-			     "', '*'])", NULL);
-	g_free (supertype);
-	
-	/* Alphebetize by name, for the sake of consistency */
-	sort[0] = g_strdup ("name");
-	sort[1] = NULL;
-
-	info_list = bonobo_activation_query (query, sort, &ev);
-	
-	if (ev._major == CORBA_NO_EXCEPTION) {
-		components_list = Bonobo_ServerInfoList_to_ServerInfo_g_list (info_list);
-		CORBA_free (info_list);
-	} else {
-		components_list = NULL;
-	}
-
-	g_free (query);
-	g_free (sort[0]);
-
-	CORBA_exception_free (&ev);
-
-	return components_list;
-}
 /**
  * gnome_vfs_mime_set_default_action_type:
  * @mime_type: a const char * containing a mime type, e.g. "image/png".
@@ -722,24 +465,12 @@ gnome_vfs_mime_id_matches_application (const char *id, GnomeVFSMimeApplication *
 	return gnome_vfs_mime_application_has_id (application, id);
 }
 
-static gint
-gnome_vfs_mime_id_matches_component (const char *iid, Bonobo_ServerInfo *component)
-{
-	return strcmp (component->iid, iid);
-}
-
 static gint 
 gnome_vfs_mime_application_matches_id (GnomeVFSMimeApplication *application, const char *id)
 {
 	return gnome_vfs_mime_id_matches_application (id, application);
 }
 
-static gint 
-gnome_vfs_mime_component_matches_id (Bonobo_ServerInfo *component, const char *iid)
-{
-	return gnome_vfs_mime_id_matches_component (iid, component);
-}
-
 /**
  * gnome_vfs_mime_id_in_application_list:
  * @id: an application id.
@@ -760,27 +491,6 @@ gnome_vfs_mime_id_in_application_list (const char *id, GList *applications)
 		 (GCompareFunc) gnome_vfs_mime_application_matches_id) != NULL;
 }
 
-/**
- * gnome_vfs_mime_id_in_component_list:
- * @iid: a component iid.
- * @components: a #GList * whose nodes are #Bonobo_ServerInfos, such as the
- * result of gnome_vfs_mime_get_short_list_components().
- * 
- * Check whether a component iid is in a list of #Bonobo_ServerInfos.
- * 
- * Return value: %TRUE if a component whose iid matches @iid is in @components.
- *
- * Deprecated: 
- */
-gboolean
-gnome_vfs_mime_id_in_component_list (const char *iid, GList *components)
-{
-	return g_list_find_custom
-		(components, (gpointer) iid,
-		 (GCompareFunc) gnome_vfs_mime_component_matches_id) != NULL;
-	return FALSE;
-}
-
 /**
  * gnome_vfs_mime_id_list_from_application_list:
  * @applications: a #GList * whose nodes are GnomeVFSMimeApplications, such as the
@@ -810,31 +520,6 @@ gnome_vfs_mime_id_list_from_application_list (GList *applications)
 	return result;
 }
 
-/**
- * gnome_vfs_mime_id_list_from_component_list:
- * @components: a #GList * whose nodes are #Bonobo_ServerInfos, such as the
- * result of gnome_vfs_mime_get_short_list_components().
- * 
- * Create a list of component iids from a list of #Bonobo_ServerInfos.
- * 
- * Return value: a new #GList where each #Bonobo_ServerInfo in the original
- * list is replaced by a char * with the component's iid. The original list is
- * not modified.
- *
- * Deprecated:
- */
-GList *
-gnome_vfs_mime_id_list_from_component_list (GList *components)
-{
-	GList *list = NULL;
-	GList *node;
-
-	for (node = components; node != NULL; node = node->next) {
-		list = g_list_prepend 
-			(list, g_strdup (((Bonobo_ServerInfo *)node->data)->iid));
-	}
-	return g_list_reverse (list);
-}
 
 /**
  * gnome_vfs_mime_add_application_to_short_list:
@@ -923,42 +608,6 @@ gnome_vfs_mime_add_component_to_short_list (const char *mime_type,
 	return GNOME_VFS_ERROR_DEPRECATED_FUNCTION;
 }
 
-/**
- * gnome_vfs_mime_remove_component_from_list:
- * @components: a #GList * whose nodes are #Bonobo_ServerInfos, such as the
- * result of gnome_vfs_mime_get_short_list_components().
- * @iid: The iid of a component to remove from @components.
- * @did_remove: If non-NULL, this is filled in with %TRUE if the component
- * was found in the list, %FALSE otherwise.
- * 
- * Remove a component specified by iid from a list of #Bonobo_ServerInfos.
- * 
- * Return value: The modified list. If the component is not found, the list will 
- * be unchanged.
- *
- * Deprecated:
- */
-GList *
-gnome_vfs_mime_remove_component_from_list (GList *components, 
-					   const char *iid,
-					   gboolean *did_remove)
-{
-	GList *matching_node;
-	
-	matching_node = g_list_find_custom 
-		(components, (gpointer)iid,
-		 (GCompareFunc) gnome_vfs_mime_component_matches_id);
-	if (matching_node != NULL) {
-		components = g_list_remove_link (components, matching_node);
-		gnome_vfs_mime_component_list_free (matching_node);
-	}
-
-	if (did_remove != NULL) {
-		*did_remove = matching_node != NULL;
-	}
-	return components;
-}
-
 /**
  * gnome_vfs_mime_remove_component_from_short_list:
  * @mime_type: a const char * containing a mime type, e.g. "application/x-php".
@@ -1111,6 +760,7 @@ gnome_vfs_mime_application_copy (GnomeVFSMimeApplication *application)
 	result->priv->icon = g_strdup (application->priv->icon);
 	result->priv->exec = g_strdup (application->priv->exec); 
 	result->priv->binary_name = g_strdup (application->priv->binary_name);
+	result->priv->path = g_strdup (application->priv->path); 
 	result->priv->supports_uris = application->priv->supports_uris;
 	result->priv->startup_notification = application->priv->startup_notification;
 	result->priv->startup_wm_class = g_strdup (application->priv->startup_wm_class);
@@ -1137,6 +787,7 @@ gnome_vfs_mime_application_free (GnomeVFSMimeApplication *application)
 			g_free (priv->icon);
 			g_free (priv->exec);
 			g_free (priv->binary_name);
+			g_free (priv->path);
 			g_free (priv->startup_wm_class);
 		}
 		g_free (priv);
@@ -1168,9 +819,6 @@ gnome_vfs_mime_action_free (GnomeVFSMimeAction *action)
 	case GNOME_VFS_MIME_ACTION_TYPE_APPLICATION:
 		gnome_vfs_mime_application_free (action->action.application);
 		break;
-	case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT:
-		CORBA_free (action->action.component);
-		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -1192,20 +840,6 @@ gnome_vfs_mime_application_list_free (GList *list)
 	g_list_free (list);
 }
 
-/**
- * gnome_vfs_mime_component_list_free:
- * @list: a #GList of #Bonobo_ServerInfo * to be freed.
- * 
- * Frees lists of #Bonobo_ServerInfo * (as returned from functions such
- * as @gnome_vfs_get_all_components).
- */
-void
-gnome_vfs_mime_component_list_free (GList *list)
-{
-	g_list_foreach (list, (GFunc) CORBA_free, NULL);
-	g_list_free (list);
-}
-
 /**
  * gnome_vfs_mime_application_new_from_id:
  * @id: a const char * containing an application id.
@@ -1264,6 +898,9 @@ gnome_vfs_mime_action_launch (GnomeVFSMimeAction *action,
 
 /**
  * gnome_vfs_mime_action_launch_with_env:
+ * @action: the #GnomeVFSMimeAction to launch.
+ * @uris: parameters for the #GnomeVFSMimeAction.
+ * @envp: the environment to use for the action.
  *
  * Same as gnome_vfs_mime_action_launch() except that the
  * application or component viewer will be launched with
@@ -1281,10 +918,6 @@ gnome_vfs_mime_action_launch_with_env (GnomeVFSMimeAction *action,
 				       GList              *uris,
 				       char              **envp)
 {
-	GnomeVFSResult result;
-	char **argv;
-	int argc;
-
 	g_return_val_if_fail (action != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 	g_return_val_if_fail (uris != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
@@ -1296,27 +929,6 @@ gnome_vfs_mime_action_launch_with_env (GnomeVFSMimeAction *action,
 			 		 uris, envp);
 					 
 	case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT:
-	
-		result = expand_component_parameters (action->action.component,
-					              uris, &argc, &argv);
-					    
-		if (result != GNOME_VFS_OK) {
-			return result;
-		}
-		
-		if (!g_spawn_async (NULL /* working directory */,
-	                            argv,
-        	                    envp,
-                	            G_SPAWN_SEARCH_PATH /* flags */,
-                        	    NULL /* child_setup */,
-				    NULL /* data */,
-	                            NULL /* child_pid */,
-        	                    NULL /* error */)) {
-			g_strfreev (argv);
-			return GNOME_VFS_ERROR_LAUNCH;
-		}
-		g_strfreev (argv);
-		
 		return GNOME_VFS_OK;		
 	
 	default:
@@ -1359,6 +971,9 @@ gnome_vfs_mime_application_launch (GnomeVFSMimeApplication *app,
 
 /**
  * gnome_vfs_mime_application_launch_with_env:
+ * @app: the #GnomeVFSMimeApplication to launch.
+ * @uris: parameters for the #GnomeVFSMimeApplication.
+ * @envp: the environment to use for the application.
  *
  * Same as gnome_vfs_mime_application_launch() except that
  * the application will be launched with the given environment.
@@ -1381,7 +996,7 @@ gnome_vfs_mime_application_launch_with_env (GnomeVFSMimeApplication *app,
 	
 	while (uris != NULL) {
 		
-		result = expand_application_parameters (app->priv->exec, &uris,
+		result = expand_application_parameters (app, &uris,
 							&argc, &argv);
 		
 		if (result != GNOME_VFS_OK) {
@@ -1395,7 +1010,7 @@ gnome_vfs_mime_application_launch_with_env (GnomeVFSMimeApplication *app,
 			}
 		}
 		
-		if (!g_spawn_async (NULL /* working directory */,
+		if (!g_spawn_async (app->priv->path,  /* working directory */
 				    argv,
 				    envp,
 				    G_SPAWN_SEARCH_PATH /* flags */,
@@ -1453,7 +1068,7 @@ expand_macro_single (char macro, const char *uri)
 }
 
 static void
-expand_macro (char macro, GString *exec, GList **uri_list)
+expand_macro (char macro, GString *exec, GnomeVFSMimeApplication *application, GList **uri_list)
 {
 	GList *uris = *uri_list;
 	char *expanded;
@@ -1495,26 +1110,46 @@ expand_macro (char macro, GString *exec, GList **uri_list)
 				}
 			}
 			break;
+		case 'i':
+			if (application->priv->icon) {
+				g_string_append (exec, "--icon ");
+				g_string_append (exec, application->priv->icon);
+			}
+			break;
+		case 'c':
+			if (application->name) {
+				g_string_append (exec, application->name);
+			}
+			break;
+		case 'k':
+			if (application->priv->desktop_file_path) {
+				g_string_append (exec, application->priv->desktop_file_path);
+			}
+		case 'm': /* deprecated */
+			break;
+		case '%':
+			g_string_append_c (exec, '%');
+			break;
 	}
 
 	*uri_list = uris;
 }
 
 static GnomeVFSResult
-expand_application_parameters (const char     *exec,
+expand_application_parameters (GnomeVFSMimeApplication *application,
 			       GList         **uris,
 			       int            *argc,
 			       char         ***argv)		   
 {
 	GList *uri_list = *uris;
-	const char *p = exec;
+	const char *p = application->priv->exec;
 	GString *expanded_exec = g_string_new (NULL);
 
 	g_return_val_if_fail (p != NULL, GNOME_VFS_ERROR_PARSE);
 
 	while (*p) {
 		if (p[0] == '%' && p[1] != '\0') {
-			expand_macro (p[1], expanded_exec, uris);
+			expand_macro (p[1], expanded_exec, application, uris);
 			p++;
 		} else {
 			g_string_append_c (expanded_exec, *p);
@@ -1566,21 +1201,23 @@ print_macro_expansion (char **argv, GnomeVFSResult res)
 }
 
 static void
-test_exec_array (const char **execs, GList *uris)
+test_exec_array (GList *apps, GList *uris)
 {
-	int argc, i;
+	int argc;
 	char **argv;
+	GList *app;
 
-	for (i = 0; execs[i] != NULL; i++)
+	for (app = apps; app != NULL; app = app->next)
 	{
+		GnomeVFSMimeApplication *application = app->data;
 		GList *l = uris;
 
-		print_expansion_data (uris, execs[i]);
+		print_expansion_data (uris, application->priv->exec);
 		while (l != NULL) {
 			GnomeVFSResult res;
 
 			res = expand_application_parameters
-					(execs[i], &l, &argc, &argv);
+					(application, &l, &argc, &argv);
 			print_macro_expansion (argv, res);
 			g_strfreev (argv);
 		}
@@ -1609,119 +1246,50 @@ test_exec_macro_expansion (void)
 				"test %u",
 				"test %U",
 				NULL };
+	const char **p;
+
+	GList* applications = NULL;
+	GnomeVFSMimeApplication *application, *app;
+
+	application = g_new0 (GnomeVFSMimeApplication, 1);
+	application->priv = g_new0 (GnomeVFSMimeApplicationPrivate, 1);
+	application->id = g_strdup ("foobar.desktop");
+	aplication->name = g_strdup ("foobar");
+	application->priv->icon = g_strdup ("icon.png");
+
+	for (p = local; p ; p++) {
+		app = gnome_vfs_mime_application_copy (application);
+		g_free (app->priv->exec);
+		app->priv->exec = g_strdup (*p);
+		applications = g_list_prepend(applications, app);
+	}
 
 	uris = g_list_append (uris, "file:///home/test/test1.txt");
 	uris = g_list_append (uris, "file:///home/test/test2.txt");
-	test_exec_array (local, uris);
-	
+	test_exec_array (applications, uris);
+
+	gnome_vfs_mime_application_list_free (applications);
+	applications = NULL;
+
+	for (p = remote; p ; p++) {
+		app = gnome_vfs_mime_application_copy (application);
+		g_free (app->priv->exec);
+		app->priv->exec = g_strdup (*p);
+		applications = g_list_prepend (applications, app);
+	}
+
 	uris = g_list_append (uris, "http://www.test.org/test1.txt");
 	uris = g_list_append (uris, "http://www.test.org/test2.txt");
-	test_exec_array (remote, uris);
+	test_exec_array (applications, uris);
+
+	gnome_vfs_mime_application_list_free (applications);
 
 	g_list_free (uris);
+
+	gnome_vfs_mime_application_free (application);
 }
 #endif
 
-static GnomeVFSResult
-expand_component_parameters (gpointer                 action,
-			     GList                   *uris,
-			     int                     *argc,
-			     char                  ***argv)		   
-{
-	Bonobo_ServerInfo *server = NULL;
-	GConfClient *client;
-	char *command = NULL;
-	char **c_argv, **r_argv;
-	int c_argc, max_r_argc;
-	int i, c;
-	gboolean added_arg;
-
-	if (!gconf_is_initialized ()) {
-		if (!gconf_init (0, NULL, NULL)) {
-			return GNOME_VFS_ERROR_INTERNAL;
-		}
-	}
-	
-	client = gconf_client_get_default ();
-	g_return_val_if_fail (client != NULL, GNOME_VFS_ERROR_INTERNAL);
-	
-	command = gconf_client_get_string (client, GCONF_DEFAULT_VIEWER_EXEC_PATH, NULL);
-	g_object_unref (client);
-		
-	if (command == NULL) {
-		g_warning ("No default component viewer set\n");
-		return GNOME_VFS_ERROR_INTERNAL;
-	}
-		
-	server = (Bonobo_ServerInfo *) action;
-		
-	if (!g_shell_parse_argv (command,
-				 &c_argc,
-				 &c_argv,
-				 NULL)) {
-		return GNOME_VFS_ERROR_PARSE;
-	}
-	g_free (command);
-
-	/* figure out how many parameters we can max have */
-	max_r_argc = g_list_length (uris) + c_argc + 1;
-	r_argv = g_new0 (char *, max_r_argc + 1);
-
-	added_arg = FALSE;
-	i = 0;
-	for (c = 0; c < c_argc; c++) {
-		/* replace %s with the uri parameters */
-		if (strcmp (c_argv[c], "%s") == 0) {
-			while (uris != NULL) {
-				r_argv[i] = g_strdup (uris->data);
-				i++;
-				uris = uris->next;
-			}
-			added_arg = TRUE;
-			
-		/* replace %c with the component iid */
-		} else if (server != NULL && strcmp (c_argv[c], "%c") == 0) {
-			r_argv[i++] = g_strdup (server->iid);
-			added_arg = TRUE;
-			
-		/* otherwise take arg from command */
-		} else {
-			r_argv[i++] = g_strdup (c_argv[c]);
-		}
-	}
-	g_strfreev (c_argv);
-	
-	/* if there is no %s or %c, append the parameters to the end */
-	if (!added_arg) {
-		while (uris != NULL) {
-			r_argv[i] = g_strdup (uris->data);
-			i++;
-			uris = uris->next;
-		}
-	}		
-	
-	*argv = r_argv;
-	*argc = i;
-
-	return GNOME_VFS_OK;
-}
-
-static GList *
-Bonobo_ServerInfoList_to_ServerInfo_g_list (Bonobo_ServerInfoList *info_list)
-{
-	GList *retval;
-	int i;
-	
-	retval = NULL;
-	if (info_list != NULL && info_list->_length > 0) {
-		for (i = 0; i < info_list->_length; i++) {
-			retval = g_list_prepend (retval, Bonobo_ServerInfo_duplicate (&info_list->_buffer[i]));
-		}
-		retval = g_list_reverse (retval);
-	}
-
-	return retval;
-}
 
 static GList *
 copy_str_list (GList *string_list)
@@ -1852,6 +1420,9 @@ gnome_vfs_mime_application_new_from_desktop_id (const char *id)
 	app->priv->binary_name = g_strdup (argv[0]);
 	g_strfreev (argv);
 
+	app->priv->path = g_key_file_get_string (key_file, DESKTOP_ENTRY_GROUP,
+						 "Path", NULL);
+
 	app->requires_terminal = g_key_file_get_boolean
 			(key_file, DESKTOP_ENTRY_GROUP, "Terminal", &err);
 	if (err) {
@@ -1895,6 +1466,22 @@ exit:
 	return app;
 }
 
+static gboolean
+uri_is_local (const char *uri)
+{
+	char *scheme;
+	gboolean local;
+
+	local = FALSE;
+	
+	scheme = gnome_vfs_get_uri_scheme (uri);
+	if (scheme != NULL) {
+		local = (strcmp (scheme, "file") == 0);
+		g_free (scheme);
+	}
+	return local;
+}
+
 /**
  * gnome_vfs_mime_application_get_default_application_for_uri:
  * @mime_type: a const char * containing a mime type, e.g. "application/x-php".
@@ -1913,35 +1500,19 @@ gnome_vfs_mime_get_default_application_for_uri (const char *uri,
 {
 	GList *applications, *l;
 	GnomeVFSMimeApplication *app = NULL;
-	char *scheme;
 	gboolean local;
 	
 	g_return_val_if_fail (uri != NULL, NULL);
 	g_return_val_if_fail (mime_type != NULL, NULL);
-	
-	scheme = gnome_vfs_get_uri_scheme (uri);
-	if (scheme != NULL) {
-		local = (strcmp (scheme, "file") == 0);
-	} else {
-		return NULL;
-	}
-	g_free (scheme);
-	
-	app = gnome_vfs_mime_get_default_application (mime_type);
-	if (local || app == NULL || gnome_vfs_mime_application_supports_uris (app)) {
-		return app;
-	} else {
-		gnome_vfs_mime_application_free (app);
-		app = NULL;
-	}
 
+	local = uri_is_local (uri);
+	
 	applications = gnome_vfs_mime_get_all_desktop_entries (mime_type);
 
 	for (l = applications; l != NULL; l = l->next) {
 		app = gnome_vfs_mime_application_new_from_id (l->data);
 
-		if (app != NULL)
-		{
+		if (app != NULL) {
 			if (local || gnome_vfs_mime_application_supports_uris (app)) {
 				break;
 			} else {
@@ -1977,19 +1548,12 @@ gnome_vfs_mime_get_all_applications_for_uri (const char *uri,
 {
 	GList *applications, *l, *result = NULL;
 	GnomeVFSMimeApplication *app;
-	char *scheme;
 	gboolean local;
 	
 	g_return_val_if_fail (uri != NULL, NULL);
 	g_return_val_if_fail (mime_type != NULL, NULL);
 	
-	scheme = gnome_vfs_get_uri_scheme (uri);
-	if (scheme != NULL) {
-		local = (strcmp (scheme, "file") == 0);
-	} else {
-		return NULL;
-	}
-	g_free (scheme);
+	local = uri_is_local (uri);
 
 	applications = gnome_vfs_mime_get_all_desktop_entries (mime_type);
 
@@ -2013,7 +1577,7 @@ gnome_vfs_mime_get_all_applications_for_uri (const char *uri,
 }
 
 /**
- * gnome_vfs_mime_application_get_desktop_file_path:
+ * gnome_vfs_mime_application_get_desktop_id:
  * @app: a #GnomeVFSMimeApplication.
  *
  * Returns the identifier of the desktop entry.
@@ -2133,7 +1697,7 @@ gnome_vfs_mime_application_get_icon (GnomeVFSMimeApplication *app)
  *
  * Returns the program to execute, possibly with arguments
  * and parameter variables, as specified by the
- * <ulink url="http://www.freedesktop.org/standards/desktop-entry-spec">
+ * <ulink url="http://standards.freedesktop.org/desktop-entry-spec/">
  * Desktop Entry Specification</ulink>.
  *
  * Return value: the command line to execute.
